/*
  This program replaces Sex (boy, girl) with Treat (control, exptal) in the program
08 Reliability 2 trials+sex.sas. It simulates extra error on the first trial (e.g., 
due to familiarization), and individual responses in the exptal group, consisting 
partly of individual responses correlated with the subject's true ability (true 
&dep) via a value for covarfactor, which is the slope of the regression line 
for individual responses vs ability. Sample values for covarfactor are derived via 
the covariance between ability and individual responses, and from the slope of the 
line of individual responses vs ability derived from the random-effect solution.
  Modify the program as shown in the proc mixed step to play with the random effects.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
*c=control, e=exptal;
%let dep=VO2max; *the dependent variable;
%let meanc=380; *true grand mean control in the first trial, back transforms to exp(380/100)=44.7;
%let meane=380; *true grand mean exptal in the first trial, same as meanc, if assume kids well randomized to control and exptal;
%let sdc=10; *true between-subject SD;
%let sde=10; *true between-subject SD;
%let tem=3; *typical (standard) error of measurement;
*%let teme=3; *typical (standard) error of measurement;
%let deltameanc=1; *change in mean Trial 2 minus Trial 1, e.g., placebo effect;
%let deltameane=5; *change in mean Trial 2 minus Trial 1;
%let extraerror1=0; *more error on Trial 1, e.g., due to familiarization;
*%let extraerror1e=0; *more error on Trial 1, e.g., due to familiarization;
%let extraerror2c=0; *more error on Trial 2, e.g., due to individual responses;
%let extraerror2e=4; *more error on Trial 2, e.g., due to individual responses;
%let covarfactor=-0.5; *fraction or multiple of subject's true SD contributing to individual responses in exptal group;
%let ssizec=20; *sample size;
%let ssizee=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for SDs and effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
Treat="control";
do ChildID=1 to &ssizec;
  True&dep=&meanc+&sdc*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameanc+&tem*rannor(0)+&extraerror2c*rannor(0);
  output;
  end;
Treat="exptal";
do ChildID=&ssizec+1 to &ssizec+&ssizee;
  True&dep=&meane+&sde*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameane+&tem*rannor(0)+&extraerror2e*rannor(0)+&covarfactor*(True&dep-&meane);
  output;
  end;

title "Raw data, first 10 observations (which will miss the exptal)";
proc print data=dat1(obs=10);
format True&dep Obsvd&dep 5.&decdep;
run;

title "Simple stats";
proc means maxdec=&decdep;
var True&dep Obsvd&dep;
class Treat Trial;
run;

*generate means and SDs for plotting;
proc sort data=dat1;
by Treat Trial;

proc means noprint data=dat1;
var Obsvd&dep;
by Treat Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1(in=a) means1;
*trick to get colored plots for individual values;
if a then do;
  if Treat="control" then Obsvd&dep.con=Obsvd&dep;
  else Obsvd&dep.exp=Obsvd&dep;
  end;
run;

ods graphics / reset width=8cm height=11cm imagemap attrpriority=none;
title "Line diagram of individual values, and means and SDs";
proc sgplot data=datplot;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
scatter x=Trial y=Mean/filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Treat groupdisplay=cluster clusterwidth=0.1 name="points"; *duplication of points to get legend;
series x=Trial y=Obsvd&dep.con/lineattrs=(pattern=solid color=lightskyblue) group=ChildID;
series x=Trial y=Obsvd&dep.exp/lineattrs=(pattern=solid color=pink) group=ChildID;
series x=Trial y=Mean/lineattrs=(pattern=solid color=black)
  group=Treat groupdisplay=cluster clusterwidth=0.1;
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Treat groupdisplay=cluster clusterwidth=0.1;
xaxis values=(1,2) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
legenditem type=line name="linec"/lineattrs=(color=lightskyblue) label="control"; 
legenditem type=line name="linee"/lineattrs=(color=pink) label="exptal";
keylegend "points" "linec" "linee";
run;

*Set up dummy variables for estimating extra variance in Trial 1 and indiv responses in Trial 2;
data dat2;
set dat1;
XvarTrial1=0; *for extra error on the first trial;
if Trial=1 then XvarTrial1=1;
XvarConTrial2=0; *for individual responses in control group;
if Treat="control" and Trial=2 then XvarConTrial2=1;
XvarExpTrial2=0; *for individual responses in exptal group;
if Treat="exptal" and Trial=2 then XvarExpTrial2=1;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Controlled-trial mixed model. Control, Exptal: Sample size=&ssizec, &ssizee; True mean=&meanc, &meane; true between SD=&sdc, &sde";
title2 "Delta mean Trial2-1=&deltameanc, &deltameane; Typical error=&tem; Extra error Trial1=&extraerror1; Extra error Trial2=&extraerror2c, &extraerror2e";
title3 "Extra error exptal Trial2 due to true ability = &covarfactor*&sde";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Treat Trial; *always make the order here the same in the model statement;
model Obsvd&dep=Treat*Trial/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
random Int XvarExpTrial2/subject=ChildID type=un s cl alpha=&alpha group=Treat;
parms 100 0 0  100 0 5  10;
*This model allows for extra error on Trial 1, but needs a large sample size:;
*random XvarTrial1/subject=ChildID;
*random Int XvarExpTrial2/subject=ChildID type=un s cl alpha=&alpha group=Treat;
*parms 0 100 0 0  100 0 5  10;
*Star off the above and unstar the repeated below to allow for the most general covparms,
 but without estimation of individual responses. Disregard the analysis of the covparms;
*repeated Trial/subject=ChildID type=un group=Treat;
estimate "Trial 2-1 cont" Treat*Trial -1 1  0 0/cl alpha=&alpha;
estimate "Trial 2-1 expt" Treat*Trial 0 0  -1 1/cl alpha=&alpha;
estimate "Trial 2-1 expt-cont" Treat*Trial 1 -1  -1 1/cl alpha=&alpha;
lsmeans Treat*Trial/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title4 "Class levels";
proc print data=clev;
run;

title4 "Covparms (random effects)";
title5 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;

data cov1 covboot;
length CovParm $ 12;
set cov;
DegFree=2*ZValue**2; *Degrees of freedom;
LagEst=lag(Estimate);
LagSE=lag(StdErr);
if covparm="UN(1,1)" then covparm="ChildID";
if covparm="UN(2,2)" then covparm="IndivResp";
if covparm="UN(2,1)" then do;
  covparm="CovFactor";
  Est=Estimate; SE=StdErr;
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2; *this was too approximate;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
array a Estimate Lower Upper; 
if CovParm ne "CovFactor" then do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
if Estimate ne 0 then output cov1;
run;

title4 "Covparm variances as SD; covariance as slope";
title5 "CovFactor is the effect of true ability on individual response as a slope; true value=&covarfactor";
title6 "True indiv. resp.=sqrt([&covarfactor*&sde]^2+&extraerror2e^2)";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;

*make macro variables for 5th and 95th percentiles;
data _null_;
call symput('pctlow',&alpha*100/2);
call symput('pctupp',100-&alpha*100/2);

proc univariate noprint data=covboot;
var EstimateBoot Estimate;
output out=covboot1(drop=d) mean=d Estimate median=EstimateBoot pctlpre=X pctlpts=&pctlow,&pctupp
 pctlname=Lower Upper;
by Group CovParm;
run;

data covboot2;
set covboot1(rename=(XLower=Lower XUpper=Upper));
CLpmApprox=(Upper-Lower)/2;

data cov1a;
set cov1;
if index(CovParm,"CovFactor");

data cov2;
merge cov1a covboot2;
by Group CovParm;

title5 "CovFactor with an extra decimal place, and bootstrapped estimate and confidence limits";
title6 "CovFactor is the effect of true ability on individual response as a slope; true value=&covarfactor";
proc print data=cov2;
var CovParm Group Estimate EstimateBoot CLpmApprox Lower Upper Alpha DegFree;
format Estimate EstimateBoot Lower Upper CLpmApprox 5.2 CLtd 5.2 DegFree 5.1; 
where CovParm="CovFactor";
run;

title4 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title4 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title4 "Least-squares means";
proc print data=lsm1;
var Effect Treat Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title4 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Treat _Treat Trial _Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title4 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Treat;
run;
ods graphics / reset;

title4 "Outliers (Standardized residual >&tvalue)";
title5 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title4 "Random-effects solution, first 20 observations";
title5 "View the whole dataset to see the values of the exptal group";
proc print data=solr(obs=20);
format Estimate StdErrPred Lower Upper 5.&deceff;
where Estimate ne 0; *suppress the irrelevant non-contributing values;
run;

*plot individual responses vs ability;
*a plot is not produced if the variance for individual responses is negative;
data abil;
set solr;
if Effect="Intercept";
if Estimate ne 0 and StdErrPred;
keep ChildID Estimate Treat;

*proc print data=solr;run;

data indresp;
set solr;
if Effect="XvarExpTrial2";
if Estimate ne 0 and StdErrPred;
keep ChildID Effect Estimate Treat;

*proc print;run;

data abilindresp;
merge abil(rename=(Estimate=Ability)) indresp(in=a rename=(Estimate=IndivResp));
by Treat ChildID;
if a;
*if Effect ne ""; *delete control subjects;

title4 "Relative ability and individual responses from solution for random effects";
title5 "First 10 observations";
proc print data=abilindresp(obs=10);
format Ability IndivResp 5.&deceff;
run;

*proc sort;
*by Effect;

ods graphics / reset width=13cm height=13cm imagemap attrpriority=none;
title4 "Scatterplot of individual responses vs relative ability";
proc sgplot data=abilindresp;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(dash solid dot);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
refline 0/axis=x lineattrs=(pattern=dot thickness=1 color=black);
reg x=Ability y=IndivResp/nomarkers lineattrs=(thickness=1) group=Effect;
scatter x=Ability y=IndivResp/filledoutlinedmarkers markerattrs=(size=12) group=Effect;
yaxis label="Relative individual response" labelpos=top;
xaxis label="Relative ability";
by Treat;
run;

data est;

ods select none;
title4 "Slope of the above line (individual responses vs relative ability) via simple regression";
title5 "True slope=&covarfactor; Uncertainty provided by the regression is not shown, because too unrealistically precise";
title6 "No scatterplot or slope are produced if individual responses have negative SD";
proc mixed data=abilindresp covtest cl alpha=&alpha nobound;
model IndivResp=Ability/ddfm=sat solution cl residual alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Slope" Ability 1/cl alpha=&alpha;
ods output estimates=est;
by Treat Effect;
run;
ods select all;

*data step to stop getting an error when est is empty;
data est1;
set est;
keep Effect Label Estimate;
format Estimate 5.2;

proc print data=est1;
run;






